/* This is "heat12.tm", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation for the heat transfer problem
   described in Chapter 17 of the text.  It is based on "heat6.c", but
   has been modified so that the function "simulate" can be called by
   Mathematica via MathLink.  This was done by adding a template for
   "simulate" and replacing the "main" function. */

/* This program can be translated into a C program by using "mprep". */


#define MAXSIZE 1001    /* The maximum number of segments that can be */
                        /* simulated.  This is all that must be modified */
                        /* to change the maximum size of the simulation. */


/* This template describes the interface of "simulate" to MathLink.  A
   template is required for every function that Mathematica is to
   call. */

:Begin:
:Function:      simulate
:Pattern:       simulate[diffusivity_Real, length_Real,
                         left_Real, right_Real, rod_Real,
                         duration_Real, segs_Integer]
:Arguments:     {diffusivity, length, left, right, rod,
                 duration, segs}
:ArgumentTypes: {Real, Real, Real, Real, Real,
                 Real, Integer}
:ReturnType:    Real
:End:


/* This function models heat transfer in a rod of uniform temperature
   that is insulated along its lengths and has a heat source applied
   to its ends.  The rod's thermal "diffusivity" in cm^2/sec and
   "length" in cm are given, as are the temperature of the "rod", the
   "left" heat source, and the "right" heat source in degrees Celsius.

   The function returns the approximate temperature of the center of
   the rod in degrees Celsius "duration" seconds after the heat sources
   are applied.

   The implementation is based on a finite-element model with "segs"
   elements. */

double simulate (double diffusivity, double length,
                 double left, double right, double rod,
                 double duration, int segs)
{

  double delta;         /* Length in seconds of each simulated time step */

  double cur[MAXSIZE],  /* Temperatures at the ends of each of the "segs" */
                        /* segments, where cur[0] is the temperature of */
                        /* the left heat source and cur[segs+1] is the */
                        /* temperature of the right heat source. */
 
         nxt[MAXSIZE];  /* Used to temporarily hold the new temperatures */
                        /* of "cur" during the simulation. */

  int i;                /* Steps through "cur" and "nxt". */


  /* Compute the length of the simulated time step. */

  delta = (length*length/(segs*segs)) / (2*diffusivity);


  /* Initialize "cur" to reflect the temperatures at time zero. */

  cur[0] = left;
  for (i = 1; i < segs; i = i+1) {
    cur[i] = rod;
  }
  cur[segs] = right;


  /* Simulate the passage of time.  During each iteration, compute the
     new segment boundary temperatures in terms of the old segment
     boundary temperatures. */

  while (duration > 0) {

    for (i = 1; i < segs; i = i+1) {
      nxt[i] = (cur[i-1] + cur[i+1]) / 2;
    }

    for (i = 1; i < segs; i = i+1) {
      cur[i] = nxt[i];
    }

    duration = duration - delta;
  }


  /* Return the temperature of the center segment boundary. */

  return(cur[segs/2]);

}



/* This is the version of "main" that must be used if MathLink is to
   communicate with the program. */

int main (int argc, char *argv[]) {
  return(MLMain(argc, argv));
}
