/* This is "heat8.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation and test driver for the
   heat transfer problem described in Chapter 17 of the text. */


#include <stdio.h>
#include <stdlib.h>

#define MAXSIZE 1001    /* The maximum number of segments that can be */
                        /* simulated.  This is all that must be modified */
                        /* to change the maximum size of the simulation. */



/* This function initializes "cur" to reflect the segment
   boundary temperatures at time zero.  There are "segs" segments.
   The temperature of the "left" heat source, the "right" heat source,
   and the "rod" are given. */

void initRod (double cur[], int segs,
              double left, double right, double rod)
{
  int i;                /* Steps through "cur". */

  cur[0] = left;
  for (i = 1; i < segs; i = i+1) {
    cur[i] = rod;
  }
  cur[segs] = right;
}



/* This function modifies "cur" to reflect the passage of one tick
   of simulated time.  There are "segs" segments. */

void nextRod (double cur[], int segs)
{

  double nxt[MAXSIZE];  /* Used to temporarily hold the new temperatures */
                        /* of "cur" during the simulation. */
  
  int i;                /* Steps through "cur" and "nxt". */

  for (i = 1; i < segs; i = i+1) {
    nxt[i] = (cur[i-1] + cur[i+1]) / 2;
  }
  
  for (i = 1; i < segs; i = i+1) {
    cur[i] = nxt[i];
  }

}  



/* This function models heat transfer in a rod of uniform temperature
   that is insulated along its lengths and has a heat source applied
   to its ends.  The rod's thermal "diffusivity" in cm^2/sec and
   "length" in cm are given, as are the temperature of the "rod", the
   "left" heat source, and the "right" heat source in degrees Celsius.

   The function returns the approximate temperature of the center of
   the rod in degrees Celsius "duration" seconds after the heat sources
   are applied.

   The implementation is based on a finite-element model with "segs"
   elements. */

double simulate (double diffusivity, double length,
                 double left, double right, double rod,
                 double duration, int segs)
{

  double delta;         /* Length in seconds of each simulated time step */

  double cur[MAXSIZE];  /* Temperatures at the ends of each of the six */
                        /* segments, where cur[0] is the temperature of */
                        /* the left heat source and cur[segs+1] is the */
                        /* temperature of the right heat source. */


  /* Initialize "cur" and compute the length of the simulated
     time step. */

  initRod(cur, segs, left, right, rod);
  delta = (length*length/(segs*segs)) / (2*diffusivity);


  /* Simulate the passage of time.  During each iteration, compute the
     new segment boundary temperatures in terms of the old segment
     boundary temperatures. */

  while (duration > 0) {
    nextRod(cur, segs);
    duration = duration - delta;
  }


  /* Return the temperature of the center segment boundary. */  

  return(cur[segs/2]);

}



/* Test driver for "simulate". */

void main (void)
{

  double C, length, left,   /* Constants specified by the user. */
         right, rod, time,
         temp;

  int segs;                 /* Size of finite-element simulation. */

  FILE *inFile;             /* Pointer to input file. */


  /* Obtain a file pointer to the file "heat-input.txt".  If the
     file cannot be opened for reading, print an explanatory
     message and exit the program immediately. */

  inFile = fopen("heat-input.txt", "r");
  if (!inFile) {
      printf("Could not open heat-input.txt\n");
      exit(1);
  }


  /* Read constants from "heat-input.txt" using "fscanf". */

  fscanf(inFile, "%lf", &C);
  fscanf(inFile, "%lf", &length);
  fscanf(inFile, "%lf%lf%lf", &left, &rod, &right);
  fscanf(inFile, "%lf", &time);
  fscanf(inFile, "%d", &segs);


  /* Close "heat-input.txt". */

  fclose(inFile);


  /* Calculate and display the result. */

  temp = simulate(C, length, left, right, rod, time, segs);
  printf("Temperature at center is %g\n", temp);

}
